package net.wanji.utc.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.vo.PhaseLockVO;
import net.wanji.utc.vo.PlanSendVO;
import net.wanji.utc.vo.schedulesend.ScheduleSendVO;
import net.wanji.utc.vo.timeplan.TimePlanVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static net.wanji.utc.common.constant.Constants.*;

/**
 * 控制指令接口
 *
 * @author wuxiaokai
 * @date 2022/11/15 13:21:10
 */
@Slf4j
@RestController
@RequestMapping("/controlCommand")
@Api(value = "控制指令接口", description = "控制指令接口", tags = "控制指令接口")
public class ControlCommandController {

    @Value("${signal.mock}")
    private boolean mock;

    @Autowired
    private ControlCommandService hkControlCommandService;

    /**
     * 信号机方案下发
     *
     * @param planSendVO 方案下发VO
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "方案下发-基础方案下发", notes = "方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public <T> Result<T> schemeSend(@RequestBody PlanSendVO planSendVO) throws Exception {
        return hkControlCommandService.planSend(planSendVO);
    }

    /**
     * todo
     * 相位配时下发?
     * 下发信号机方案?
     *
     * @param code      信号机id
     * @param timePlans 配时方案实体类
     * @return {@link Result}<{@link T}>
     * @throws InterruptedException 中断异常
     */
    @ApiOperation(value = "相位配时下发-相位的绿灯、绿闪、行闪、黄灯、红灯配时下发", notes = "相位配时下发-相位的绿灯、绿闪、行闪、黄灯、红灯配时下发")
    @PostMapping("/phaseTimingSend")
    public <T> Result<T> phaseTimingSend(@RequestParam String code, @RequestBody List<TimePlanVO> timePlans) throws InterruptedException {
        return hkControlCommandService.phaseTimingSend(code, timePlans);
    }

    /**
     * 下发信号机计划
     *
     * @param planSendVO 计划下发VO
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public <T> Result<T> planSend(@RequestBody PlanSendVO planSendVO) throws Exception {
        return hkControlCommandService.planSend(planSendVO);
    }

    /**
     * 全红控制
     *
     * @param code    信号机id
     * @param command 指令
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "全红控制-路口全红控制/恢复", notes = "全红控制-路口全红控制/恢复")
    @PostMapping("/allRedControl")
    public <T> Result<T> allRedControl(@RequestParam String code, @RequestParam Integer command) throws Exception {
        return hkControlCommandService.setSignalControl(code, command, ALL_RED_CONTROL);
    }

    /**
     * 黄闪控制
     *
     * @param code    信号id
     * @param command 命令
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "黄闪控制-路口黄闪控制/恢复", notes = "黄闪控制-路口黄闪控制/恢复")
    @PostMapping("/yellowLightControl")
    public <T> Result<T> yellowLightControl(@RequestParam String code, @RequestParam Integer command) throws Exception {
        return hkControlCommandService.setSignalControl(code, command, YELLOW_LIGHT_CONTROL);
    }

    /**
     * 关灯控制
     *
     * @param code    信号id
     * @param command 命令
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "关灯控制-路口关灯控制/开灯", notes = "关灯控制-路口关灯控制/开灯")
    @PostMapping("/closeLightControl")
    public <T> Result<T> closeLightControl(@RequestParam String code, @RequestParam Integer command) throws Exception {
        return hkControlCommandService.setSignalControl(code, command, CLOSE_LIGHT_CONTROL);
    }

    /**
     * 步进控制
     *
     * @param code    信号id
     * @param command 命令
     * @param stepNum 一步num
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @PostMapping("/stepControl")
    public <T> Result<T> stepControl(@RequestParam String code,
                                     @RequestParam Integer command,
                                     @RequestParam int stepNum) throws Exception {
        return hkControlCommandService.stepControl(code, command, stepNum);
    }

    /**
     * 恢复时间表
     *
     * @param code 信号id
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/recoverSchedule")
    public <T> Result<T> recoverSchedule(@RequestParam String code) throws Exception {
        return hkControlCommandService.recoverSchedule(code);
    }

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public <T> Result<T> scheduleSend(@RequestBody ScheduleSendVO scheduleSendVO) throws Exception {
        return hkControlCommandService.scheduleSend(scheduleSendVO);
    }

    /**
     * 锁定控制
     *
     * @param phaseLock 相位锁定实体类
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public <T> Result<T> lockControl(@RequestBody PhaseLockVO phaseLock) throws Exception {
        return hkControlCommandService.lockControl(phaseLock);
    }
}
